"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.symbolToType = exports.filterTypes = void 0;
const tsutils_1 = require("tsutils");
const total_functions_1 = require("total-functions");
/**
 * Throws away non-object types (string, number, boolean, etc) because we don't check those for readonly -> mutable assignment.
 */
exports.filterTypes = (rawDestinationType, rawSourceType) => {
    // TODO if the destination is a union containing _all_ mutable types, we're assigning to mutable.
    // TODO if the destination contains a mix of mutable and readonly types, we don't know if we're assigning
    // to mutable unless we can narrow down which destination (parameter) types apply to the given source (argument) types.
    // TODO if the sourceType is a union containing _any_ readonly types, we're assigning from readonly.
    const filteredDestinationTypes = tsutils_1.unionTypeParts(rawDestinationType).filter((t) => tsutils_1.isObjectType(t));
    const destinationType = filteredDestinationTypes.length === 1
        ? total_functions_1.get(filteredDestinationTypes, 0)
        : undefined;
    const filteredSourceTypes = tsutils_1.unionTypeParts(rawSourceType).filter((t) => tsutils_1.isObjectType(t));
    const sourceType = filteredSourceTypes.length === 1 ? total_functions_1.get(filteredSourceTypes, 0) : undefined;
    return {
        destinationType,
        sourceType,
    };
};
exports.symbolToType = (
// eslint-disable-next-line @typescript-eslint/ban-types
s, checker) => {
    const declarations = s.getDeclarations() || [];
    // TODO: How to choose declaration when there are multiple?
    const declaration = declarations.length > 1 ? undefined : total_functions_1.get(declarations, 0);
    return declaration !== undefined
        ? checker.getTypeAtLocation(declaration)
        : undefined;
};
